import { visitParents } from "unist-util-visit-parents";
import { fromHtml } from "hast-util-from-html";
import { renderEditor, renderCodeBlock } from "prism-code-editor/ssr";
import { numLines } from "prism-code-editor";
import { languages, tokenizeText, highlightTokens } from "prism-code-editor/prism";
const parseValue = (value, numLines2) => {
  if (value[0] == '"' || value[0] == "'") value = value.slice(1, -1);
  if (value[0] == "{") return parseRanges(value, numLines2);
  if (!value || value == "true") return true;
  if (value == "false") return false;
  if (value == "null") return null;
  if (!isNaN(+value)) return +value;
  return value;
};
const parseRanges = (ranges, numLines2) => {
  let pattern = /(\d+)(?:\s*-\s*(\d+))?/g;
  let match;
  let result = /* @__PURE__ */ new Set();
  while (match = pattern.exec(ranges)) {
    let start = Math.max(1, +match[1]);
    let end = Math.min(+match[2] || start, numLines2);
    while (start <= end) result.add(start++);
  }
  return result;
};
const getRanges = (props, prop) => {
  let ranges = props[prop];
  delete props[prop];
  if (ranges instanceof Set) return ranges;
};
const parseMeta = (meta, numLines2) => {
  const result = {};
  const pattern = /([^\s"'{}=]+)(?:\s*=\s*("[^"]*"|'[^']*'|\{[^}]*\}|[^\s"'{}=]+))?/g;
  let match;
  while (match = pattern.exec(meta)) {
    let [, prop, value] = match;
    result[prop] = parseValue(value || "", numLines2);
  }
  return result;
};
const createEditor = (props) => {
  return renderEditor(props);
};
const createCodeBlock = (props) => {
  const highlight = getRanges(props, "highlight");
  const ins = getRanges(props, "ins");
  const del = getRanges(props, "del");
  if (highlight || ins || del) {
    props.addLineClass = (line) => {
      if (highlight?.has(line)) return "highlighted";
      if (ins?.has(line)) return "inserted";
      if (del?.has(line)) return "deleted";
      return "";
    };
  }
  return renderCodeBlock(props);
};
const rehypePrismCodeEditor = ({
  editorsOnly,
  defaultEditorProps,
  defaultCodeBlockProps,
  customRenderer,
  inline,
  silenceWarnings
} = {}) => {
  return (tree) => {
    const nodes = [];
    const spans = [];
    visitParents(tree, { type: "element", tagName: "code" }, (node, ancestors) => {
      const [grandParent, parent] = ancestors.slice(-2);
      const children = node.children;
      const text = children[0];
      if (children[1] || text.type != "text") return;
      if (parent.tagName != "pre") {
        if (inline) spans.push([node, text.value]);
      } else if (!parent.children[1]) {
        nodes.push([node, parent, grandParent, text.value.slice(0, -1)]);
      }
    });
    nodes.forEach(([codeEl, pre, parent, code]) => {
      const properties = codeEl.properties;
      const meta = codeEl.data?.meta || properties.metastring || "";
      const props = parseMeta(meta, numLines(code));
      const isEditor = props.editor;
      if (editorsOnly && isEditor == null) return;
      props.value = code;
      delete props.editor;
      let className = properties.className;
      if (Array.isArray(className)) className = className.join(" ");
      const lang = (typeof className == "string" && /\blanguage-(\S+)/.exec(className)?.[1] || "text").replace(/\b\+\+/g, "pp").replace(/\b#/g, "sharp");
      props.language = lang;
      if (!languages[lang] && !silenceWarnings) {
        console.warn(
          `rehype-prism-code-editor: Unregistered language '${lang}' found in code block. Syntax highlighting will be disabled.`
        );
      }
      const renderFunc = isEditor ? createEditor : createCodeBlock;
      const merged = {
        ...isEditor ? defaultEditorProps : defaultCodeBlockProps,
        ...props
      };
      const html = customRenderer ? customRenderer(merged, renderFunc, !!isEditor) : renderFunc(merged);
      const children = parent.children;
      children[children.indexOf(pre)] = fromHtml(html, { fragment: true }).children[0];
    });
    spans.forEach(([codeEl, text]) => {
      const langStart = text.lastIndexOf("{:");
      if (langStart < 0 || text.slice(-1) != "}") return;
      const lang = text.slice(langStart + 2, -1);
      const code = text.slice(0, langStart);
      const grammar = languages[lang];
      if (!grammar) {
        if (!silenceWarnings)
          console.warn(
            `rehype-prism-code-editor: Unregistered language '${lang}' found in inline code. Highlighting is skipped.`
          );
        return;
      }
      const properties = codeEl.properties;
      const className = properties.className;
      const langClass = "language-" + lang;
      const tokens = tokenizeText(code, grammar);
      inline.tokenizeCallback?.(tokens, lang);
      if (Array.isArray(className)) className.push(langClass);
      else if (typeof className == "string") properties.className = [className, langClass];
      else properties.className = langClass;
      codeEl.children = fromHtml(highlightTokens(tokens), { fragment: true }).children;
    });
  };
};
export {
  rehypePrismCodeEditor
};
//# sourceMappingURL=index.js.map
